// Breakout.cpp : Breakout Game

// Header file includes
#include <Windows.h>	// Win32 API
#include <d3d9.h>		// Direct3D
#include <d3dx9.h>		// D3D extensions -- texture and sprite utilities
#include <cmath>
#include "Breakout.h"   //Base class
#include "resource.h"   //App resources
#include <dsound.h>
#include "DirectSoundLab.h"
#include "SoundPlayer.h"

// Forward declarations of functions included in this code module:

LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
void drawBricks(ID3DXSprite* pSprite, IDirect3DTexture9* pBrick, GameObj* obj,SoundPlayer*,SoundPlayer*);
void drawPaddle(ID3DXSprite* pSprite, IDirect3DTexture9* pPaddle, GameObj* obj);
void drawBall(ID3DXSprite* pSprite, IDirect3DTexture9* pBall, GameObj* obj);
void collision(GameObj* obj,SoundPlayer*,SoundPlayer*);
void restart(GameObj* pWnd);

// Constants
#define SPEED 10
#define BALL 56
#define PADDLE 55
#define BALL_SIZE 22
#define BRICK_H 25
#define BRICK_W 50
#define PADDLE_H 18
#define PADDLE_W 121
#define BALL_FILE "ball.png"
#define BLOCK_FILE "block.png"
#define PADDLE_FILE "paddle.png"
#define GEN_HIT "genHit.wav"
#define BRICK_HIT "brickHit.wav"
#define SCREEN_H 532
#define SCREEN_W 762
#define FREQUENCY 1.519958
#define VOLUME -2559
#define SAFE_RELEASE( p ) if((p))(p)->Release();(p) = NULL  //Macro to delete D3D pointers

INT WINAPI WinMain( HINSTANCE, HINSTANCE, LPSTR, INT )  //Program Entry Point
{
	// Register the window class
	WNDCLASSEX wc = { 
		sizeof(WNDCLASSEX),   
		CS_CLASSDC, 
		MsgProc, 
		0L, 
		0L, 
		GetModuleHandle(NULL), 
		LoadIcon(GetModuleHandle(NULL), (LPCTSTR)IDI_BREAKOUT), 
		NULL, 
		NULL, 
		(LPCTSTR)IDC_BREAKOUT, 
		"Breakout", 
		LoadIcon(GetModuleHandle(NULL), (LPCTSTR)IDI_SMALL) 
	};
	RegisterClassEx( &wc );

	// Create the application's window
	HWND hWnd = CreateWindow( "Breakout", "Breakout", 
		WS_OVERLAPPEDWINDOW, 100, 100, 770, 600,
		NULL, NULL, wc.hInstance, NULL );

	HRESULT hr = S_OK;
	HRESULT hr1 = D3DERR_DEVICELOST;
	HRESULT hr2 = D3DERR_INVALIDCALL;
	HRESULT hr3 = D3DERR_NOTAVAILABLE;
	HRESULT hr4 = D3DERR_OUTOFVIDEOMEMORY;



// define an IDirect3D9 pointer and an IDirect3DDevice9 pointer.
	IDirect3D9* pD3D = NULL;
	IDirect3DDevice9* pDevice = NULL;

// create the D3D object using Direct3DCreate9
	pD3D = Direct3DCreate9( D3D_SDK_VERSION );
	hr = ( pD3D != NULL ) ? S_OK : E_OUTOFMEMORY;


	if( SUCCEEDED( hr ) )
	{
		// Set up the structure that defines how we want to use D3D:

		// Set up a D3DPRESENT_PARAMETERS struct for a windowed app that uses the current display settings.
		D3DPRESENT_PARAMETERS d3dpp = { 0 };
		
		// Specify windowed mode
		d3dpp.Windowed = TRUE;					
		
		//Setup swap effect
		d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;

		//Setup D3D format
		d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
		
		// Create the D3D Device:

		// Initialize the IDirect3DDevice9 pointer by calling IDirect3D9::CreateDevice.
	   	hr = pD3D->CreateDevice(
			D3DADAPTER_DEFAULT,					// choose the default device if there's more than one
			D3DDEVTYPE_HAL,						// Specify we want a hardware renderer
			hWnd,								// Attach D3D to our lame little window
			D3DCREATE_SOFTWARE_VERTEXPROCESSING,// We want the GPU to do vertex shading
			&d3dpp,								// The structure that we filled out above
			&pDevice );							// Pointer to the device pointer--this is an "out param."
		 if(hr==hr1)
			MessageBox(hWnd,"D3DERR_DEVICELOST","ble",NULL);
		 else if(hr==hr2)
			MessageBox(hWnd,"D3DERR_INVALIDCALL","ble",NULL);
		 else if(hr==hr3)
			MessageBox(hWnd,"D3DERR_NOTAVAILABLE","ble",NULL);
		 else if(hr==hr4)
			MessageBox(hWnd,"D3DERR_OUTOFVIDEOMEMORY","ble",NULL);
	}

	SoundPlayer* pPlay1;
	SoundPlayer* pPlay2;
		
	SoundPlayer player1(hWnd,GEN_HIT);
	pPlay1=&player1;
	SoundPlayer player2(hWnd,BRICK_HIT);
	pPlay2=&player2;
	//Defien Array of Game Objects
	GameObj objArray[57];
	SetWindowLongPtr(hWnd,GWLP_USERDATA,LONG_PTR(&objArray[0]));
	//initialize Paddle
	objArray[PADDLE].setPos(D3DXVECTOR3(((SCREEN_W/2)-(PADDLE_W/2)), (SCREEN_H-PADDLE_H), NULL));
	objArray[PADDLE].setTop((SCREEN_H-PADDLE_H));
	objArray[PADDLE].setLeft(((SCREEN_W/2)-(PADDLE_W/2)));
	objArray[PADDLE].setBottom(objArray[PADDLE].getTop()+PADDLE_H);
	objArray[PADDLE].setRight(objArray[PADDLE].getLeft()+PADDLE_W);

	//intitialize Ball data;
	float aTest=0;
	do{
		aTest=float(rand()%180);
	}while(((aTest<95)&&(aTest>85))||((aTest<185)&&(aTest>175))||((aTest<5)&&(aTest>355)));
	objArray[BALL].setAngle(aTest);
	objArray[BALL].setPos(D3DXVECTOR3( ((SCREEN_W/2)-(BALL_SIZE/2)), (objArray[PADDLE].getTop()-BALL_SIZE)-2, NULL));
	objArray[BALL].setTop((objArray[PADDLE].getTop()-BALL_SIZE)-2);
	objArray[BALL].setLeft(((SCREEN_W/2)-(BALL_SIZE/2)));
	objArray[BALL].setBottom(objArray[BALL].getTop()+BALL_SIZE);
	objArray[BALL].setRight(objArray[BALL].getLeft()+BALL_SIZE);

	//Define Texture objects
	IDirect3DTexture9* pBrick = NULL;
	IDirect3DTexture9* pPaddle = NULL;
	IDirect3DTexture9* pBall = NULL;
	if( SUCCEEDED(hr) )
	{
		// Load textures from file using D3DXCreateTextureFromFile and check to make sure all load correctly
		hr = D3DXCreateTextureFromFile(   //load brick
			pDevice,
			BLOCK_FILE,
			&pBrick );
		if( SUCCEEDED(hr) )
		{
			hr = D3DXCreateTextureFromFile(   //load paddle
			pDevice,
			PADDLE_FILE,
			&pPaddle );
			if( SUCCEEDED(hr) )
			{
				hr = D3DXCreateTextureFromFile(  //load ball
				pDevice,
				BALL_FILE,
				&pBall );
			}
		}
	}

	//Sprite Object
	ID3DXSprite* pSprite = NULL;
	if( SUCCEEDED( hr ) )
	{
		// Create the sprite
		hr = D3DXCreateSprite( pDevice, &pSprite );
	}

	if( SUCCEEDED( hr ) )
	{
		// Show the application window
		ShowWindow( hWnd, SW_SHOW );

		// MAIN MESSAGE LOOP
		MSG msg = { 0 };											
		while( msg.message != WM_QUIT )
		{
			Sleep( 10 );										
			while( PeekMessage( &msg, NULL, 0, 0, TRUE ) )		
			{																
				TranslateMessage( &msg );						 
				DispatchMessage( &msg );						
			}													

			//-------RENDERING CODE------------//

			//Clear the Back buffer and prepare it to be drawn on
			pDevice->Clear( 0, NULL, D3DCLEAR_TARGET, 0x00000000, 0.f, 0L );
			pDevice->BeginScene();

			//Begin the process of drawing sprites to Hardware
			pSprite->Begin( D3DXSPRITE_ALPHABLEND );

			//Call drawing function to display all the game objects: takes all out
			//textures and the Sprite object that holds them
			drawBricks(pSprite,pBrick,&objArray[0],pPlay1,pPlay2);
			drawPaddle(pSprite,pPaddle,&objArray[PADDLE]);
			drawBall(pSprite,pBall,&objArray[BALL]);
			
			// commit the drawing to the hardware
			pSprite->End();

			// End and present the Draw Scene
			pDevice->EndScene();
			pDevice->Present( NULL, NULL, NULL, NULL );
		}

	}

	// Release all of the resource pointers we acquired

	// release the sprite pointer
	SAFE_RELEASE( pSprite );
	// release the texture pointer
	SAFE_RELEASE( pBrick );
	SAFE_RELEASE( pPaddle );
	SAFE_RELEASE( pBall );

	// release the IDirect3DDevice pointer
	SAFE_RELEASE( pDevice );

	// release the IDirect3D pointer
	SAFE_RELEASE( pD3D );

	return 0;
}



//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//


//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	GameObj* pWnd;

	switch( msg )
	{
	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		wmEvent = HIWORD(wParam); 
		// Parse the menu selections:
		switch (wmId)
		{
		case ID_FILE_NEWGAME:
			pWnd=(GameObj*)GetWindowLongPtr(hWnd,GWLP_USERDATA);
			restart(pWnd);
			break;
		case IDM_EXIT:
			DestroyWindow(hWnd);
			break;
		default:
			return DefWindowProc(hWnd, msg, wParam, lParam);
		}
		break;
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return 0;
	
	case WM_MOUSEMOVE:
		pWnd=(GameObj*)GetWindowLongPtr(hWnd,GWLP_USERDATA);
		pWnd[PADDLE].setPos(D3DXVECTOR3( float(LOWORD(lParam))-(PADDLE_W/2), pWnd[PADDLE].getTop(), NULL));
		pWnd[PADDLE].setLeft(float(LOWORD(lParam))-(PADDLE_W/2));
		pWnd[PADDLE].setRight(pWnd[PADDLE].getLeft()+PADDLE_W);
		break;
	case WM_PAINT:
		ValidateRect( hWnd, NULL );
		return 0;
	}

	return DefWindowProc( hWnd, msg, wParam, lParam );
}

//Function to draw all the game elements
void drawBricks(ID3DXSprite* pSprite, IDirect3DTexture9* pBrick, GameObj* obj,SoundPlayer* p1,SoundPlayer* p2)
{
	float x=0,y=0;  //represent our screen position
	int checkDone=0;
	D3DXVECTOR3 pos( x, y, 0);  //the 2D vector that is our screen pos.
	int index=0;
	//check for collision
	collision(obj,p1,p2);
	//Draw 5 rows of 11 columns of bricks
	for(int drawLoopY=0; drawLoopY<5; drawLoopY++)
	{
		for(int drawLoopX=0; drawLoopX<11; drawLoopX++)
		{
			if(obj[index].isHit()==false)
			{
				x=float(drawLoopX*(BRICK_W+20));
				y=float(drawLoopY*(BRICK_H+15));
				pos=D3DXVECTOR3( x, y, 0);
				pSprite->Draw( 
					pBrick,			// texture to draw
					NULL,			// source rect -- NULL to draw the whole thing
					NULL,			// sprite center--NULL for default
					&pos,			// draw position--NULL for default
					0xffffffff );	// color modulator--0xffffffff is no change in color
				obj[index].setTop(y);
				obj[index].setLeft(x);
				obj[index].setBottom(y+BRICK_H);
				obj[index].setRight(x+BRICK_W);
				obj[index].setPos(pos);
			}
			else
				checkDone++;
			index++;
		}
	}
	if(checkDone==55)
		restart(obj);
}

void drawPaddle(ID3DXSprite* pSprite, IDirect3DTexture9* pPaddle, GameObj* obj)
{
	float x=0,y=0;  //represent our screen position
	D3DXVECTOR3 pos( x, y, 0);  //the 2D vector that is our screen pos.
	pos=(*obj).getPos();
	//draw the paddle
	pSprite->Draw( 
		pPaddle,		
		NULL,			
		NULL,			
		&pos,			
		0xffffffff );
/*	(*obj).setTop(y);
	(*obj).setLeft(x);
	(*obj).setBottom(y+BRICK_H);
	(*obj).setRight(x+BRICK_W);
	(*obj).setPos(pos);*/
}

void drawBall(ID3DXSprite* pSprite, IDirect3DTexture9* pBall, GameObj* obj)
{
	float x=0,y=0;
	float pi=float(3.1415926535897932384626433832795);  //represent our screen position
	D3DXVECTOR3 pos( x, y, 0);  //the 2D vector that is our screen pos.
	//draw the ball
	pos=(*obj).getPos();
	x=cos(((*obj).getAngle()*pi)/180);
	y=-sin(((*obj).getAngle()*pi)/180);
	pos+=D3DXVECTOR3(SPEED*x,SPEED*y,NULL);
	pSprite->Draw( 
		pBall,		
		NULL,			
		NULL,			
		&pos,			
		0xffffffff );
	(*obj).setTop((*obj).getTop()+(SPEED*y));
	(*obj).setLeft((*obj).getLeft()+(SPEED*x));
	(*obj).setBottom((*obj).getBottom()+(SPEED*y));
	(*obj).setRight((*obj).getRight()+(SPEED*x));
	(*obj).setPos(pos);
}
void collision(GameObj* obj,SoundPlayer* pPlay1,SoundPlayer* pPlay2)
{
	float angle=obj[BALL].getAngle(),test;

	//check right wall
	if((SCREEN_W-obj[BALL].getRight()<SPEED)&&(SCREEN_W-obj[BALL].getRight()>0))
	{
		if(angle<90)
			obj[BALL].setAngle(180-angle);
		else
			obj[BALL].setAngle(180+(360-angle));
		obj[BALL].setPos(obj[BALL].getPos()-D3DXVECTOR3(1,NULL,NULL));
		obj[BALL].setLeft(obj[BALL].getLeft()-1);
		obj[BALL].setRight(obj[BALL].getRight()-1);
		 pPlay1->Play( float(FREQUENCY), long(VOLUME) );
	}
	
	//check left wall
	else if((obj[BALL].getLeft()<SPEED)&&(obj[BALL].getLeft()>0))
	{
		if(angle<180)
			obj[BALL].setAngle(180-angle);
		else
			obj[BALL].setAngle(360-(angle-180));
		obj[BALL].setPos(obj[BALL].getPos()+D3DXVECTOR3(1,NULL,NULL));
		obj[BALL].setLeft(obj[BALL].getLeft()+1);
		obj[BALL].setRight(obj[BALL].getRight()+1);
		 pPlay1->Play( float(FREQUENCY), long(VOLUME) );
	}

	//check top bound
	else if(obj[BALL].getTop()<SPEED)
	{
		obj[BALL].setAngle(360-angle);
		 pPlay1->Play( float(FREQUENCY), long(VOLUME) );
	}

	//ckeck bottom bound
	else if(obj[BALL].getBottom()>SCREEN_H)
	{
		//reset ball angle
		obj[BALL].setAngle(360-angle);
		 pPlay1->Play( float(FREQUENCY), long(VOLUME) );
		restart(obj);
	}

	//check for paddle collision
	else if((obj[PADDLE].getTop()-obj[BALL].getBottom()<SPEED)&&(obj[PADDLE].getTop()-obj[BALL].getBottom()>0))
	{
		if((obj[PADDLE].getLeft()<=obj[BALL].getRight())&&(obj[BALL].getLeft()<=obj[PADDLE].getRight()))
		{
				obj[BALL].setAngle(360-angle);
				obj[BALL].setPos(obj[BALL].getPos()-D3DXVECTOR3(NULL,2,NULL));
				obj[BALL].setTop(obj[BALL].getTop()-2);
				obj[BALL].setBottom(obj[BALL].getBottom()-2);
			 pPlay2->Play( float(FREQUENCY), long(VOLUME) );
		}
	}

	//check for brick collision
	else if(obj[BALL].getTop()<199)
	{
		for(int colCheck=54; colCheck>=0; colCheck--)
		{
			if(obj[colCheck].isHit()==false)
			{
				//check bottom collision
				test=obj[BALL].getTop()-obj[colCheck].getBottom();

				if((0<test)&&(test<SPEED))
				{
					if((obj[colCheck].getLeft()<=obj[BALL].getRight())&&(obj[colCheck].getRight()>=obj[BALL].getLeft()))
					{
						obj[colCheck].beenHit();
						obj[BALL].setAngle(360-obj[BALL].getAngle());
						 pPlay2->Play( float(FREQUENCY), long(VOLUME) );
						break;
					}
				}

				//check Left side collision
				test=obj[colCheck].getLeft()-obj[BALL].getRight();
				if((0<test)&&(test<SPEED))
				{
					if((obj[colCheck].getTop()<=obj[BALL].getBottom())&&(obj[colCheck].getBottom()>=obj[BALL].getTop()))
					{
						obj[colCheck].beenHit();
						if(angle<90)
							obj[BALL].setAngle(180-angle);
						else
							obj[BALL].setAngle(180+(360-angle));
						obj[BALL].setPos(obj[BALL].getPos()-D3DXVECTOR3(1,NULL,NULL));
						obj[BALL].setLeft(obj[BALL].getLeft()-1);
						obj[BALL].setRight(obj[BALL].getRight()-1);
						 pPlay2->Play( float(FREQUENCY), long(VOLUME) );
						break;
					}
				}
				//check right side collision
				test=obj[BALL].getLeft()-obj[colCheck].getRight();
				if((0<test)&&(test<SPEED))
				{
					if((obj[colCheck].getTop()<=obj[BALL].getBottom())&&(obj[colCheck].getBottom()>=obj[BALL].getTop()))
					{
						obj[colCheck].beenHit();
						if(angle<180)
							obj[BALL].setAngle(180-angle);
						else
							obj[BALL].setAngle(360-(angle-180));
						obj[BALL].setPos(obj[BALL].getPos()+D3DXVECTOR3(1,NULL,NULL));
						obj[BALL].setLeft(obj[BALL].getLeft()+1);
						obj[BALL].setRight(obj[BALL].getRight()+1);
						 pPlay2->Play( float(FREQUENCY), long(VOLUME) );
						break;
					}
				}
				//check top collision
				test=obj[colCheck].getTop()-obj[BALL].getBottom();

				if((0<test)&&(test<SPEED))
				{
					if((obj[colCheck].getLeft()<=obj[BALL].getRight())&&(obj[colCheck].getRight()>=obj[BALL].getLeft()))
					{
						obj[colCheck].beenHit();
						obj[BALL].setAngle(360-angle);
						 pPlay2->Play( float(FREQUENCY), long(VOLUME) );
						break;
					}
				}
			}
		}
	}
}
void restart(GameObj* pWnd)
{
	float aTest=0;
	do{
		aTest=float(rand()%180);
	}while(((aTest<95)&&(aTest>85))||((aTest<185)&&(aTest>175))||((aTest<5)&&(aTest>355)));
	for(int reset=0;reset<57;reset++)
	{
		pWnd[reset].resetHit();
	}
	pWnd[PADDLE].setPos(D3DXVECTOR3(((SCREEN_W/2)-(PADDLE_W/2)), (SCREEN_H-PADDLE_H), NULL));
	pWnd[PADDLE].setTop((SCREEN_H-PADDLE_H));
	pWnd[PADDLE].setLeft(((SCREEN_W/2)-(PADDLE_W/2)));
	pWnd[PADDLE].setBottom(pWnd[PADDLE].getTop()+PADDLE_H);
	pWnd[PADDLE].setRight(pWnd[PADDLE].getLeft()+PADDLE_W);

	//intitialize Ball data;
	pWnd[BALL].setAngle(aTest);
	pWnd[BALL].setPos(D3DXVECTOR3( ((SCREEN_W/2)-(BALL_SIZE/2)), (pWnd[PADDLE].getTop()-BALL_SIZE)-2, NULL));
	pWnd[BALL].setTop((pWnd[PADDLE].getTop()-BALL_SIZE)-2);
	pWnd[BALL].setLeft(((SCREEN_W/2)-(BALL_SIZE/2)));
	pWnd[BALL].setBottom(pWnd[BALL].getTop()+BALL_SIZE);
	pWnd[BALL].setRight(pWnd[BALL].getLeft()+BALL_SIZE);
}